--[[
-- external functions
+ updateAvailableMounts()
+ initializeAddon()
+ summonMount(mountType, mountSpeed, namePattern)
+ companionList = listMounts(mountType, mountSpeed, namePattern)
- mountType, speed = getBestMount()
+ getChatCmd(input)

-- internal helpers
+ n = getTableSize(tbl)
+ companionIdList = spellID2companionID(spellId)
+ companionIdList = spellID2companionID(spellIdList)
- mountType, speed = getBestMountByLocation()
- mountType, speed = getBestMountBySkill()
+ mountIdList = queryMountDB(mountType=nil(=all), mountSpeed=nil(=all), namePattern=nil(=all))

-- chat functions
+sendChatMessage(message)
~showChatHelp()
]]--

-- -------------------------------------------------------------------------------------------
Mountaneer = {
	['availableMounts']	= {},
	['mountDB']			= {},
	['spell2companion']	= {},
	['frame']			= CreateFrame('frame');
}

function Mountaneer:initializeAddon()
	if not self.frame then
		self.frame = CreateFrame('frame');
	else
		self.frame:UnregisterEvent('PLAYER_LOGIN');	-- no need reloading this too often
	end;
	
	self.frame:RegisterEvent('COMPANION_UPDATE');
	self.frame:SetScript('OnEvent', 
		function(event, arg1, ...) 
			if event == 'COMPANION_UPDATE' and arg1 == 'MOUNT' then 
				Mountaneer:updateAvailableMounts() 
			end 
		end
	);
	
	-- do it manually the first time
	self:updateAvailableMounts();
	
	self:sendChatMessage('succesfully loaded.');
end;

function Mountaneer:updateAvailableMounts()
	-- clean the cache first, so we don't get any stale data (eg. due to a skill-increase. Some mounts change speed based on skill)
	self['availableMounts']	= {};
	self['spell2companion']	= {};
	
	-- no sense updating the list if there ARE no mounts.
	if GetNumCompanions('MOUNT') == 0 then
		return;
	end;
	
	-- make a list of all spellID's of the mounts we own
	for i=1,GetNumCompanions('MOUNT') do
		_, _, spellID = GetCompanionInfo('MOUNT', i);
		self['spell2companion'][spellID] = i;
	end;
	
	-- insert mounts found into the cache in the proper category
	for catName, content in pairs(self['mountDB']) do
		self['availableMounts'][catName] = {};
		
		for _, mount in ipairs(content) do
			if self['spell2companion'][mount] then
				self['availableMounts'][catName][#(self['availableMounts'][catName])+1] = mount;
			end;
		end;
	end;
end;

function Mountaneer:sendChatMessage(message, prefix)
	if prefix == nil then
		prefix = '|cff00ff00Mountaneer:|r ';
	end;
	
	DEFAULT_CHAT_FRAME:AddMessage(prefix..message);
end;

function Mountaneer:showChatHelp()
	self:sendChatMessage('Help not yet implemented.');
end;

function Mountaneer:summonMount(mountType, mountSpeed, namePattern)
	local mountList = self:queryMountDB(mountType, mountSpeed, namePattern);
	local n = self:getTableSize(mountList);
	
	if n > 0 then
		local result = random(1, n);
		local companionId = self:spellID2companionID(mountList[result]);
		
		local link = GetSpellLink(mountList[result]);
		
		self:sendChatMessage('Summoning '..link);
		CallCompanion('MOUNT', companionId);
	else
		self:sendChatMessage('No mounts found matching your query.');
	end;
end;

function Mountaneer:listMounts(mountType, mountSpeed, namePattern)
	-- list of companionID's
	local mountList = self:queryMountDB(mountType, mountSpeed, namePattern);
	local n = self:getTableSize(mountList);
	
	self:sendChatMessage(n..' mounts matched your query:');
	if n > 0 then
		for _, spellId in pairs(mountList) do			
			self:sendChatMessage(GetSpellLink(spellId), '  ');
		end;
	end;
end;

function Mountaneer:queryMountDB(mountType, mountSpeed, namePattern)
	local returnTable = {};
 	local cats = {};
	
	if mountType == 'any' then
		-- add categories, limited by speed
		cats[#cats+1] = 'special';
		cats[#cats+1] = 'swimming';

		if mountSpeed == 'slow' or mountSpeed == 'any' then
			cats[#cats+1] = 'land-slow';
			cats[#cats+1] = 'flying-slow';
		end;
		if mountSpeed == 'fast' or mountSpeed == 'any' then
			cats[#cats+1] = 'land-fast';
			cats[#cats+1] = 'flying-fast';
		end;
		if mountSpeed == 'extreme' or mountSpeed == 'any' then
			cats[#cats+1] = 'flying-extreme';
		end;
		
	elseif mountType == 'land' then
		if mountSpeed == 'slow' or mountSpeed == 'any' then
			cats[#cats+1] = 'land-slow';
		end;
		if mountSpeed == 'fast' or mountSpeed == 'any' then
			cats[#cats+1] = 'land-fast';
		end;
	elseif mountType == 'flying' then
		if mountSpeed == 'slow' or mountSpeed == 'any' then
			cats[#cats+1] = 'flying-slow';
		end;
		if mountSpeed == 'fast' or mountSpeed == 'any' then
			cats[#cats+1] = 'flying-fast';
		end;
		if mountSpeed == 'extreme' or mountSpeed == 'any' then
			cats[#cats+1] = 'flying-extreme';
		end;
	elseif mountType == 'swimming' then
		cats[#cats+1] = 'swimming';
	elseif mountType == 'special' then
		cats[#cats+1] = 'special';
	end;
	
	-- make a complete list with all mounts from the type/speed combo's
	for i=1,#cats do
		for j=1,#(self.availableMounts[cats[i]]) do
			returnTable[#returnTable+1] = self.availableMounts[cats[i]][j];
		end;
	end;
	
	-- filter the list based on name
	if namePattern == 'random' then
		return returnTable;	-- special case, return all
	else
		local finalReturn = {};
		namePattern = string.lower(namePattern);	-- make case-insensitive
		
		for i=1,#returnTable do
			local spellId = returnTable[i];
			local mountNameCS = GetSpellInfo(spellId);
			local mountName = string.lower(mountNameCS);	-- case-insensitive
			if string.match(mountName, namePattern) then
				finalReturn[#finalReturn+1] = spellId;
			end;
		end;
		
		return finalReturn;
	end;
end;

function Mountaneer:spellID2companionID(input)
	if type(input) == 'table' then
		local returnTbl = {};
		
		for i=1,#input do
			returnTbl[#returnTbl+1] = self['spell2companion'][input[i]];
		end;
		
		return returnTbl;
	else
		return self['spell2companion'][input];
	end;
end;

function Mountaneer:getTableSize(tbl)
	local n = 0;
	
	for _ in pairs(tbl) do
		n = n+1;
	end;
	
	return n;
end;

function Mountaneer:getChatCmd(input)
	local parts = {strsplit(' ', input)};

	--[[
		[actionType] [mountType] [mountSpeed] [namePattern]
	]]--
	
	local actionType, mountType, mountSpeed, namePattern = nil, nil, nil, nil;
	local depth = 1;
	local querySize = self:getTableSize(parts);
	local doneParsing = false;
	
	-- figure out what is what in the string
	if querySize == 0 then
		actionType = 'help';
	else
		-- [actionType]
		if parts[depth] == 'help' or parts[depth] == 'max' or parts[depth] == 'init' then
			actionType = parts[depth];
			doneParsing = true;
		elseif parts[depth] == 'list' or parts[depth] == 'summon' then
			actionType = parts[depth];
			depth = depth+1;
		else
			actionType = 'summon';	-- default value
		end;
		
		-- [mountType]
		if(not doneParsing and querySize >= depth) then
			if parts[depth] == 'land' or parts[depth] == 'flying' or parts[depth] == 'swimming' or parts[depth] == 'special' then
				mountType = parts[depth];
				depth = depth+1;
			else
				mountType = 'any';	-- default value
			end;
		end;

		-- [mountSpeed]
		if(not doneParsing and querySize >= depth) then
			if parts[depth] == 'slow' or parts[depth] == 'rare' then
				mountSpeed = 'slow';
				depth = depth+1;
			elseif parts[depth] == 'fast' or parts[depth] == 'epic' then
				mountSpeed = 'fast';
				depth = depth+1;
			elseif parts[depth] == 'extreme' then
				mountSpeed = 'extreme';
				depth = depth+1;
			elseif parts[depth] == 'any' then
				mountSpeed = 'any';
				depth = depth+1;
			else
				mountSpeed = 'any';	-- default value
			end;
		end;

		-- [namePattern]
		if(not doneParsing and querySize >= depth) then
			local nameParts = '';
			for i=depth,querySize do
				if nameParts == '' then
					nameParts = parts[i];
				else
					nameParts = nameParts..' '..parts[i];
				end;
			end;
			
			namePattern = nameParts;
		else
			namePattern = 'random';
		end;
	end;
	
	
	-- perform actual action
	if actionType == 'help' then
		self:showChatHelp();
	
	elseif actionType == 'init' then
		self:initializeAddon();
	
	elseif actionType == 'max' then
		self:sendChatMessage('/mount max has not been implemented yet');
		
		--mountType, mountSpeed = self:getBestMount();
		--self:summonMount(mountType, mountSpeed, nil);
	
	elseif actionType == 'list' then
		self:listMounts(mountType, mountSpeed, namePattern);
	
	elseif actionType == 'summon' then
		self:summonMount(mountType, mountSpeed, namePattern);
	end;
end;

SLASH_MOUNTANEER1 = '/mount';
SlashCmdList['MOUNTANEER'] = function(input) Mountaneer:getChatCmd(input) end;

Mountaneer.frame:RegisterEvent('PLAYER_LOGIN');
Mountaneer.frame:SetScript('OnEvent', function() Mountaneer:initializeAddon() end);